syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface WrenchStamped
/// \brief Message for a time stamped wrench value

import "time.proto";
import "wrench.proto";

message WrenchStamped
{
  // Time when the data was captured
  required Time time       = 1;
  required Wrench wrench   = 2;
}
