syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Image
/// \brief Message for an image


message Image
{
  required uint32 width         = 1; // Image width (number of columns)
  required uint32 height        = 2; // Image height (number of rows)
  required uint32 pixel_format  = 3; // Corresponds to Image::PixelFormat enum
  required uint32 step          = 4; // Full row length in bytes
  // repeated uint32 data          = 5; // Actual data, size if (step * rows)
  required bytes data          = 5; // Actual data, size if (step * rows)
}
